import { Observable } from '@nativescript/core';
/**
 * Describes a task function.
 *
 * @param {TState} [state] The optional value to submit to the function.
 *
 * @return {TResult} The result of the function.
 */
export declare type TaskFunc<TState, TResult, TUpdate> = (ctx: TaskContext<TState, TUpdate>) => TResult;
export declare type TaskFuncUpdate<TUpdate> = (update: {
    data: TUpdate;
}) => void;
/**
 * Describes a task context.
 */
export interface TaskContext<TState = any, TUpdate = any> {
    state: TState;
    onProgressUpdate: (dataUpdate: TUpdate) => void;
    [key: string]: any;
}
/**
 * Describes a result of a task.
 */
export interface TaskResult<TState, TResult> {
    /**
     * The result data (on success)
     */
    data?: TResult;
    /**
     * The error (if occurred)
     */
    error?: any;
    /**
     * The submitted value.
     */
    state?: TState;
}
export interface TaskUpdate<TUpdate> {
    /**
     * The update data (on update)
     */
    data: TUpdate;
}
/**
 * List of task states.
 */
export declare enum TaskStatus {
    Starting = 0,
    /**
     * The task has been initialized but has not yet been invoked.
     */
    Created = 1,
    /**
     * The task completed due to an unhandled exception.
     */
    Faulted = 2,
    /**
     * The task completed execution successfully.
     */
    RanToCompletion = 3,
    /**
     * The task is running but has not yet completed.
     */
    Running = 4,
    /**
     * The task has been scheduled for execution but has not yet begun executing.
     */
    WaitingToRun = 5
}
/**
 * A task.
 */
export declare class Task<TState, TResult, TUpdate> extends Observable {
    /**
     * Stores the function to invoke.
     */
    protected readonly _FUNC: TaskFunc<TState, TResult, TUpdate>;
    protected readonly _FUNC_UPDATE?: TaskFuncUpdate<TUpdate>;
    /**
     * Stores the error of the last execution.
     */
    protected _error: any;
    /**
     * Stores the current task state.
     */
    protected _status: TaskStatus;
    /**
     * Initializes a new instance of that class.
     *
     * @param {TaskFunc<TState, TResult>} func The function to invoke.
     */
    constructor(func: TaskFunc<TState, TResult, TUpdate>, funcUptade?: TaskFuncUpdate<TUpdate>);
    static initGlobalWorker(options?: {
        moduleWorker: boolean;
    }): void;
    static finishGlobalWorker(): void;
    /**
     * Creates a new task.
     *
     * @param {TaskFunc<TResult>} func The function to invoke.
     *
     * @return {Task<TResult>} The new task.
     */
    static newTask<TResult, TUpdate>(func: TaskFunc<any, TResult, TUpdate>, onProgressUpdate?: TaskFuncUpdate<TUpdate>): Task<any, TResult, TUpdate>;
    /**
     * Creates and starts a new task.
     *
     * @param {TaskFunc<TResult>} func The function to invoke.
     * @param {TState} [state] The optional value / object for the execution.
     *
     * @return {Promise<TResult>} The promise.
     */
    static start<TState, TResult, TUpdate>(func: TaskFunc<TState, TResult, TUpdate>, options?: {
        state?: TState;
        onProgressUpdate?: TaskFuncUpdate<TUpdate>;
        attachToContextFunctions?: {};
    }): Promise<TaskResult<TState, TResult>>;
    /**
     * Gets the error of the last execution.
     */
    get error(): any;
    /**
     * Gets the underyling function to invoke.
     */
    get func(): TaskFunc<TState, TResult, TUpdate>;
    /**
     * Gets the underyling upudate function to invoke.
     */
    get funcUpdate(): TaskFuncUpdate<TUpdate> | undefined;
    /**
     * Starts the task.
     *
     * @param {TState} [state] The optional value to submit to the function.
     *
     * @return {Promise<TaskResult<TState, TResult>>} The promise.
     */
    start<TState>(options?: {
        state?: TState;
        attachToContextFunctions?: {};
    }): Promise<TaskResult<TState, TResult>>;
    /**
     * Gets the current status.
     */
    get status(): TaskStatus;
    /**
     * Updates the error value.
     *
     * @param {TaskStatus} newValue The new value.
     *
     * @return {boolean} Property change has been raised for 'error' property or not.
     */
    protected updateError(newValue: any): boolean;
    /**
     * Updates the current status.
     *
     * @param {TaskStatus} newValue The new value.
     *
     * @return {boolean} Property change has been raised for 'status' property or not.
     */
    protected updateStatus(newValue: TaskStatus): boolean;
}
