// The MIT License (MIT)
//
// Copyright (c) Marcel Joachim Kloubert <marcel.kloubert@gmx.net>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
import { Observable } from '@nativescript/core';
import { JSONfn } from './JSONfn';
/**
 * List of task states.
 */
export var TaskStatus;
(function (TaskStatus) {
    TaskStatus[TaskStatus["Starting"] = 0] = "Starting";
    /**
     * The task has been initialized but has not yet been invoked.
     */
    TaskStatus[TaskStatus["Created"] = 1] = "Created";
    /**
     * The task completed due to an unhandled exception.
     */
    TaskStatus[TaskStatus["Faulted"] = 2] = "Faulted";
    /**
     * The task completed execution successfully.
     */
    TaskStatus[TaskStatus["RanToCompletion"] = 3] = "RanToCompletion";
    /**
     * The task is running but has not yet completed.
     */
    TaskStatus[TaskStatus["Running"] = 4] = "Running";
    /**
     * The task has been scheduled for execution but has not yet begun executing.
     */
    TaskStatus[TaskStatus["WaitingToRun"] = 5] = "WaitingToRun";
})(TaskStatus || (TaskStatus = {}));
let globalWorker = null;
/**
 * A task.
 */
export class Task extends Observable {
    /**
     * Initializes a new instance of that class.
     *
     * @param {TaskFunc<TState, TResult>} func The function to invoke.
     */
    constructor(func, funcUptade) {
        super();
        /**
         * Stores the current task state.
         */
        this._status = TaskStatus.Starting;
        if (func !== null && func !== undefined) {
            if (typeof func !== 'function') {
                throw "'func' must be a function!";
            }
        }
        this._FUNC = func;
        this._FUNC_UPDATE = funcUptade;
        this.updateStatus(TaskStatus.Created);
    }
    static initGlobalWorker(options = { moduleWorker: false }) {
        if (globalWorker == null) {
            if (options.moduleWorker === true) {
                globalWorker = new Worker('./worker');
            }
            else {
                globalWorker = new Worker('@/globalWorker');
            }
        }
    }
    static finishGlobalWorker() {
        globalWorker.terminate();
        globalWorker = null;
    }
    /**
     * Creates a new task.
     *
     * @param {TaskFunc<TResult>} func The function to invoke.
     *
     * @return {Task<TResult>} The new task.
     */
    static newTask(func, onProgressUpdate) {
        return new Task(func, onProgressUpdate);
    }
    /**
     * Creates and starts a new task.
     *
     * @param {TaskFunc<TResult>} func The function to invoke.
     * @param {TState} [state] The optional value / object for the execution.
     *
     * @return {Promise<TResult>} The promise.
     */
    static start(func, options) {
        return Task.newTask(func, options?.onProgressUpdate).start(options);
    }
    /**
     * Gets the error of the last execution.
     */
    get error() {
        return this._error;
    }
    /**
     * Gets the underyling function to invoke.
     */
    get func() {
        return this._FUNC;
    }
    /**
     * Gets the underyling upudate function to invoke.
     */
    get funcUpdate() {
        return this._FUNC_UPDATE;
    }
    /**
     * Starts the task.
     *
     * @param {TState} [state] The optional value to submit to the function.
     *
     * @return {Promise<TaskResult<TState, TResult>>} The promise.
     */
    start(options) {
        let me = this;
        return new Promise((resolve, reject) => {
            let completed = (err, data) => {
                if (err) {
                    reject({
                        error: err,
                        state: options?.state,
                    });
                }
                else {
                    resolve({
                        data: data,
                        state: options?.state,
                    });
                }
                me.updateError(err);
            };
            switch (me._status) {
                case TaskStatus.Created:
                case TaskStatus.Faulted:
                case TaskStatus.RanToCompletion:
                    break;
                default:
                    completed(new Error(`Cannot start while in '${TaskStatus[me._status]}' state!`));
                    return;
            }
            try {
                me.updateStatus(TaskStatus.WaitingToRun);
                if (globalWorker == null) {
                    Task.initGlobalWorker();
                }
                globalWorker.onmessage = function (msg) {
                    // worker.terminate();
                    let result = msg.data;
                    if (result) {
                        result = JSON.parse(result);
                    }
                    if (result?.onProgressUpdate === true) {
                        if (me._FUNC_UPDATE) {
                            me._FUNC_UPDATE({ data: result.dataUpdate });
                        }
                    }
                    else {
                        me.updateStatus(TaskStatus.RanToCompletion);
                        completed(null, result);
                    }
                };
                globalWorker.onerror = function (err) {
                    me.updateStatus(TaskStatus.Faulted);
                    completed(err);
                };
                let func;
                if (me._FUNC) {
                    func = {};
                    func.body = JSONfn.stringify(me._FUNC);
                    if (options.attachToContextFunctions && Object.keys(options.attachToContextFunctions).length > 0) {
                        const functionsToAttach = Object.keys(options.attachToContextFunctions).reduce((resultObject, keyFunction) => {
                            resultObject[keyFunction] = JSONfn.stringify(options.attachToContextFunctions[keyFunction]);
                            return resultObject;
                        }, {});
                        func.attachToContextFunctions = functionsToAttach;
                    }
                }
                me.updateStatus(TaskStatus.Running);
                globalWorker.postMessage(JSON.stringify({
                    func: func,
                    state: options?.state,
                }));
            }
            catch (e) {
                console.log(e);
                me.updateStatus(TaskStatus.Faulted);
                completed(e);
            }
        });
    }
    /**
     * Gets the current status.
     */
    get status() {
        return this._status;
    }
    /**
     * Updates the error value.
     *
     * @param {TaskStatus} newValue The new value.
     *
     * @return {boolean} Property change has been raised for 'error' property or not.
     */
    updateError(newValue) {
        if (newValue !== this._error) {
            this._error = newValue;
            this.notifyPropertyChange('error', newValue);
            return true;
        }
        return false;
    }
    /**
     * Updates the current status.
     *
     * @param {TaskStatus} newValue The new value.
     *
     * @return {boolean} Property change has been raised for 'status' property or not.
     */
    updateStatus(newValue) {
        if (newValue !== this._status) {
            this._status = newValue;
            this.notifyPropertyChange('status', newValue);
            return true;
        }
        return false;
    }
}
//# sourceMappingURL=task.js.map